class ClickEndpoints:
    """Click API endpoints."""
    PREPARE = "prepare"
    COMPLETE = "complete"
    MERCHANT_API = "merchant/api"


class ClickNetworks:
    """Click API networks."""
    TEST_NET = "https://api.click.uz/v2/merchant"
    PROD_NET = "https://api.click.uz/v2/merchant"


class ClickActions:
    """Click API actions."""
    PREPARE = 0
    COMPLETE = 1
